#require 'continuation' rescue nil
require 'sdl'
require 'singleton'
require 'pumpkin_modern/font.rb'
require 'pumpkin_modern/input.rb'
require 'pumpkin_modern/fpstimer.rb'

class Input
  define_key SDL::Key::ESCAPE, :exit
  define_key SDL::Key::LEFT,   :left
  define_key SDL::Key::RIGHT,  :right
  define_key SDL::Key::SPACE,  :jump
end

class Task

  GameInfo = Struct.new(:key, :screen, :timer)

  def wait
    #Fiber.yield̂
    cont = nil
    xx = callcc{|cc| cont = cc}
    if xx.is_a? Continuation
      throw :pause_task, xx
    end
  end

  class Runner
    include Singleton

    def initialize
      @tasks = []
    end

    def add_task(task)
      @tasks << task
    end

    def start
      info = GameInfo.new
      info.key    = Input.new
      info.screen = SDL.set_video_mode( 640, 480, 16, SDL::SWSURFACE) #|SDL::FULLSCREEN) 
      info.timer  = FPSTimerSample.new
      
      # make fibers from each task
      fibers = []
      @tasks.each do |task| 
        task.methods.find_all{|method|
          method.to_s =~ /\Aact_/
        }.each{|method|
          fibers << [task, method]
        }
      end

      info.timer.reset

      until fibers.empty?
        # get key events
        info.key.poll
        break if info.key[:exit]

        # process each fiber 
        fibers.map! do |fiber|
          ret = nil
          case fiber
          when Array
            task, method = fiber
            ret = catch(:pause_task){
              task.__send__(method, info)
            }
          when Continuation
            ret = catch(:pause_task){
              fiber.call(info)
            }
          end
          if ret.is_a? Continuation then ret else nil end
        end.compact!

        # refresh screen
        info.timer.wait_frame do
          info.screen.update_rect(0,0,0,0)
        end
      end
    end
  end

end

class ShowFPS < Task
  def initialize
    #@font = SDL::TTF.load("image/boxfont2.ttf", 18)
    #@font = SDL::BMFont.load("image/5x5font.bmp")
    #@font = SDL::BMFont.open("image/5x5font.bmp", SDL::BMFont::TRANSPARENT)
  end

  def act_count_fps(i)
    @drawn_frames = 0
    loop do
      prev = Time.now
      prev_skip = i.timer.total_skip
      while (Time.now - prev) < 0.5
        wait
      end
      skipped = i.timer.total_skip - prev_skip
      @drawn_frames = i.timer.real_fps.to_i - skipped
      puts "#{@drawn_frames} / #{i.timer.real_fps}"
    end
  end

#  def act_render(i)
##      @font.draw_blended_utf8(i.screen, "foo", 0, 0, 
##                        255,255,255)
#    @font.write(i.screen, "#{@drawn_frames} / #{i.timer.real_fps}",
#                0, 0)
#  end
end

class BackGround < Task
  def act_render(i)
    loop do
      i.screen.fill_rect(0, 0, 640, 480, [0,0,0])
      wait
    end
  end
end

class Enemy < Task

  def initialize
    @x=rand(640-32)
    @y=rand(480-32)
  end

  def act_move2(i)
    dx=rand(3)-1
    dy=rand(3)-1
    while dx == 0 && dy == 0
      dx=rand(3)-1
      dy=rand(3)-1
    end

    loop do
      @x += dx
      if @x < 0 || @x >= 640-32 then
        dx = -dx
        @x += dx
      end
      @y += dy
      if @y < 0 || @y >= 480-32 then
        dy = -dy
        @y += dy
      end
      wait
    end
  end
  
#  def act_move1(i)
#    @x = rand(640)
#    @y = rand(480)
#    40.times{ @x += 8; wait }
#    40.times{ @y += 8; wait }
#    40.times{ @x -= 8; wait }
#    40.times{ @y -= 8; wait }
#    @alive = false
#  end

  def act_render(i)
    img = SDL::Surface.load_bmp("image/icon.bmp")
    img.set_color_key(SDL::SRCCOLORKEY, [255,255,255])
    @alive = true
    loop do
      break if not @alive
      i.screen.put(img, @x, @y)
      wait
    end
  end
end

class Player < Task

  def initialize
    @x = 0
    @y = 0
  end

  def act_holizontal(i)
    @x = 0
    loop do
      if i.key[:left]
        @x -= 8
      end
      if i.key[:right]
        @x += 8
      end
      wait
    end
  end

  DY = -50
  AY = 4
  def act_jump(i)
    @y = 400
    loop do
      if i.key.pushed? :jump
        dy = DY
        ay = AY
        while @y <= 400
          @y  += dy
          dy += ay
          wait
        end
        @y = 400
      end
      wait
    end
  end

  def act_render(i)
    img = SDL::Surface.load_bmp("image/noschar.bmp").copy_rect(0,0,32,32)
    img.set_color_key(SDL::SRCCOLORKEY, [252,254,252])
    loop do
      i.screen.put(img, @x, @y)
      wait
    end
  end
end

SDL.init( SDL::INIT_VIDEO|SDL::INIT_JOYSTICK|SDL::INIT_AUDIO)
SDL::TTF.init
runner = Task::Runner.instance
runner.add_task(BackGround.new)
15.times{runner.add_task(Enemy.new)}
runner.add_task(Player.new)
runner.add_task(ShowFPS.new)
runner.start
