module PumpkinModern

  module TextWindow

    def puts(str)
      str.to_s.scan(/.{1,#{@max_chars}}/).each do |s|
        @data << s
      end
      @data = @data.last(@lines) if @data.size > @lines
    end

    def print(str)
      @data[-1] << str.to_s
    end

    def clear
      @data = []
    end

    def render_text(screen, left, top, color = nil)
      @data.each_with_index do |line, y|
        if color
          @font.write(screen, line, left, top+y*@font.line_skip, color)
        else
          @font.write(screen, line, left, top+y*@font.line_skip )
        end
      end
    end

  end

  class DebugMessage < Task
    priority 9999
    include TextWindow
    include Singleton

    def initialize
      @data = [
        "Welcome to PumpkinModern debug console :)",
        ""
      ]
      @font = Font.load("image/5x5font.bmp") #, nil, :sfont)
      @lines = 15
      @max_chars = 640 / @font.char_width
      @status_list = []
    end

    def self.print(str)      ; self.instance.print(str)      ; end
    def self.puts(str)       ; self.instance.puts(str)       ; end
    def self.clear           ; self.instance.clear           ; end
    def self.add_status(&st) ; self.instance.add_status(&st) ; end

    def add_status(&block)
      @status_list << block
    end

    def render(i)
      status_str = @status_list.map{|st| st.call}.join(' ')
      @font.write(i.screen, status_str, 0, 0)
      render_text(i.screen, 0, @font.line_skip)
    end

  end

end
